searchNodes=[{"doc":"Functions for integrating Kafka tracing with DataStreams. Usage Because Elixir does not include a context grab bag to pass around, we use the OpenTelemetry context to store the current DataStreams pathway. If you are not using OpenTelemetry, or have a special fan in or fan out situation, you can use the respective functions that take a pathway as an argument. If you have a basic one message in one message out situation, and you have OpenTelemetry already covering your application, you can use the trace_produce/1 and trace_consume/2 functions. require OpenTelemetry.Tracer , as : Tracer alias Datadog.DataStreams.Integrations.Kafka , as : DataStreamsKafka @doc &quot;Handles a message from Kafka. Receives a message map with partition, topic, and headers.&quot; @spec handle_message ( map ( ) ) :: :ok def handle_message ( message ) do # NOTE: This does not add the recommended Kafka span attributes. Tracer . with_span &quot; \#{ message . topic } process&quot; do DataStreamsKafka . trace_consume ( message , &quot;my_consumer_group&quot; ) # Do work Tracer . with_span &quot; \#{ new_message . topic } produce&quot; do new_message |&gt; DataStreamsKafka . trace_produce ( ) |&gt; send_to_kafka ( ) end end end","ref":"Datadog.DataStreams.Integrations.Kafka.html","title":"Datadog.DataStreams.Integrations.Kafka","type":"module"},{"doc":"Traces a Kafka message being consumed. Requires the current Kafka consumer group. Uses the pathway in the current Datadog.DataStreams.Context .","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_consume/2","title":"Datadog.DataStreams.Integrations.Kafka.trace_consume/2","type":"function"},{"doc":"Traces a Kafka message being consumed. Requires the current Kafka consumer group. Do not pass the resulting pathway from this function to another call of trace_consume_with_pathway , as it will modify the pathway incorrectly.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_consume_with_pathway/3","title":"Datadog.DataStreams.Integrations.Kafka.trace_consume_with_pathway/3","type":"function"},{"doc":"Traces a Kafka message being produced. Uses the pathway in the current Datadog.DataStreams.Context . Returns a new message with the pathway encoded in the header values.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_produce/1","title":"Datadog.DataStreams.Integrations.Kafka.trace_produce/1","type":"function"},{"doc":"Traces a Kafka message being produced. Returns the new message with the pathway encoded in the header values, as well as the new pathway.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_produce_with_pathway/2","title":"Datadog.DataStreams.Integrations.Kafka.trace_produce_with_pathway/2","type":"function"},{"doc":"A general map that contains the topic, partition, and headers atoms. This matches the format of Elsa.elsa_message by default (and will work out of the box), though will need the topic and partition added if you are using standard :brod (or kpro ).","ref":"Datadog.DataStreams.Integrations.Kafka.html#t:message/0","title":"Datadog.DataStreams.Integrations.Kafka.message/0","type":"type"},{"doc":"This is a port of the data-streams-go library to Elixir. Configuration Configuration can be done via the Config module and your config/ files. We use OpenTelemetry configured values first, so if you are already using OpenTelemetry to instrument your application, your service name and environment should already be set. import Config config :opentelemetry , :resource , name : &quot;my-elixir-service&quot; , env : &quot;production&quot; If you are not using OpenTelemetry, you can set the service, environment, and primary tag via the :data_streams application like so: import Config config :data_streams , :metadata , service : &quot;my-elixir-service&quot; , env : &quot;production&quot; , primary_tag : &quot;datacenter:d1&quot; Once that is configured, you will also want to set the configuration for accessing the Datadog agent. This can be done via: import Config config :data_streams , :agent , enabled? : true , host : &quot;my-datadog-agent.local&quot; , port : 8125 By default, the agent is disabled and will not send data to Datadog . Without the host or port configured, we default to &quot;localhost:8125&quot;. For more information, view the Datadog.DataStreams.Config module. Running Once installed, if :dd_agent_streams :agent is enabled? , the Datadog.DataStreams.Aggregator will start automatically and start sending metrics. Just instrument your data pipelines! Integrations Datadog.DataStreams.Integrations.Kafka Telemetry Similar to the golang implementation, we export a couple of :telemetry metrics that can (and should) be sent to Datadog. These can be done via how ever you are sending your application telemetry metrics to Datadog. Available metrics include: datadog.datastreams.aggregator.payloads_in.count - The number of Datadog.DataStreams.Aggregator.Point s that were sent to the Datadog.DataStreams.Aggregator . datadog.datastreams.aggregator.flushed_payloads.count - The number of successful payloads sent to Datadog. datadog.datastreams.aggregator.flushed_buckets.count - The number of successful 10 second buckets sent to Datadog. datadog.datastreams.aggregator.flush_errors.count - The number of failed requests to the Datadog agent (and dropped payloads).","ref":"Datadog.DataStreams.html","title":"Datadog.DataStreams","type":"module"},{"doc":"A GenServer instance responsible for aggregating many points of data together into 10 second buckets, and then sending them to the Datadog agent. It holds many structs in its memory, looking something like this: graph TD aggregator[Datadog.DataStreams.Aggregator] aggregator --&gt; bucket[Datadog.DataStreams.Aggregator.Bucket] bucket --&gt; group[Datadog.DataStreams.Aggregator.Group] When adding data, the calling code will create a new Datadog.DataStreams.Aggregator.Point which contains all of the needed data. It will then call Elixir.Datadog.DataStreams.Aggregator.add/1 to add that point of data to the aggregator, where the aggregator will find (or create) a bucket that matches the 10 second window for the point. It will then find (or create) a group in that bucket based on the point's hash . Once the group is found, the pathway_latency and edge_latency Datadog.Sketch will be updated with the new latency. Every 10 seconds the aggregator will convert all non active (outside the 10 second window) to a Datadog.DataStreams.Payload , encode it, and send it to the Datadog agent. If there is an error sending the payload, the old payloads are still removed from memory, but the datadog.datastreams.aggregator.flush_errors.count telemetry metric is incremented.","ref":"Datadog.DataStreams.Aggregator.html","title":"Datadog.DataStreams.Aggregator","type":"module"},{"doc":"Adds new metrics to the aggregator. Note, this function will still return :ok if the aggregator is disabled. Examples iex&gt; :ok = Aggregator . add ( % Aggregator.Point { } )","ref":"Datadog.DataStreams.Aggregator.html#add/1","title":"Datadog.DataStreams.Aggregator.add/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Datadog.DataStreams.Aggregator.html#child_spec/1","title":"Datadog.DataStreams.Aggregator.child_spec/1","type":"function"},{"doc":"Sends all stored data to the Datadog agent. Examples iex&gt; :ok = Aggregator . flush ( )","ref":"Datadog.DataStreams.Aggregator.html#flush/0","title":"Datadog.DataStreams.Aggregator.flush/0","type":"function"},{"doc":"Starts a new Elixir.Datadog.DataStreams.Aggregator instance. This takes no options as it uses the global Datadog.DataStreams.Config module. It is also started by the Datadog.DataStreams.Application and should not need to be started manually.","ref":"Datadog.DataStreams.Aggregator.html#start_link/1","title":"Datadog.DataStreams.Aggregator.start_link/1","type":"function"},{"doc":"Responsible for parsing application configuration to usable chunks in the application.","ref":"Datadog.DataStreams.Config.html","title":"Datadog.DataStreams.Config","type":"module"},{"doc":"Checks if the Datadog agent is enabled. By default it is disabled. Disabling will cause the Datadog.DataStreams.Aggregator process to not start or send data to the agent. Examples iex&gt; Config . agent_enabled? false iex&gt; Application . put_env ( :data_streams , :agent , enabled? : true ) ...&gt; Config . agent_enabled? true","ref":"Datadog.DataStreams.Config.html#agent_enabled?/0","title":"Datadog.DataStreams.Config.agent_enabled?/0","type":"function"},{"doc":"Returns a full path to the Datadog agent, joining the given path string. If this configuration is not set, we default to &quot;localhost:8126&quot;. Examples iex&gt; Config . agent_url ( &quot;/info&quot; ) &quot;http://localhost:8126/info&quot; iex&gt; Application . put_env ( :data_streams , :agent , [ host : &quot;my-agent.local&quot; , port : 1234 ] ) ...&gt; Config . agent_url ( &quot;/info&quot; ) &quot;http://my-agent.local:1234/info&quot;","ref":"Datadog.DataStreams.Config.html#agent_url/1","title":"Datadog.DataStreams.Config.agent_url/1","type":"function"},{"doc":"Returns the configured environment tag for this application. First, it will try accessing the configured service for :open_telemetry . If that is not set, it tries to use the service configured for :data_streams . If that is not set, it falls back to an empty string. Note, this will not pull open telemetry attributes set via environment variables. Examples iex&gt; Application . put_env ( :opentelemetry , :resource , service : %{ env : &quot;production&quot; } ) ...&gt; Config . env ( ) &quot;production&quot; iex&gt; Application . put_env ( :data_streams , :metadata , env : &quot;staging&quot; ) ...&gt; Config . env ( ) &quot;staging&quot; iex&gt; Config . env ( ) &quot;&quot;","ref":"Datadog.DataStreams.Config.html#env/0","title":"Datadog.DataStreams.Config.env/0","type":"function"},{"doc":"Returns the configured primary tag for all Datadog.DataStreams.Pathway s. Usually this reflects a data center or some other top level partition for metrics. If this is not set, it will fall back to an empty string. Examples iex&gt; Application . put_env ( :data_streams , :metadata , primary_tag : &quot;datacenter:d1&quot; ) ...&gt; Config . primary_tag ( ) &quot;datacenter:d1&quot; iex&gt; Config . primary_tag ( ) &quot;&quot;","ref":"Datadog.DataStreams.Config.html#primary_tag/0","title":"Datadog.DataStreams.Config.primary_tag/0","type":"function"},{"doc":"Returns the configured service for this application. First, it will try accessing the configured service for :open_telemetry . If that is not set, it tries to use the service configured for :data_streams . If that is not set, it falls back to &quot;unnamed-elixir-service&quot;. Note, this will not pull open telemetry attributes set via environment variables. Examples iex&gt; Application . put_env ( :opentelemetry , :resource , service : %{ name : &quot;my-elixir-service&quot; } ) ...&gt; Config . service ( ) &quot;my-elixir-service&quot; iex&gt; Application . put_env ( :data_streams , :metadata , service : &quot;my-data-streams-service&quot; ) ...&gt; Config . service ( ) &quot;my-data-streams-service&quot; iex&gt; Config . service ( ) &quot;unnamed-elixir-service&quot;","ref":"Datadog.DataStreams.Config.html#service/0","title":"Datadog.DataStreams.Config.service/0","type":"function"},{"doc":"Logic for linking current running container id to data stream traces.","ref":"Datadog.DataStreams.Container.html","title":"Datadog.DataStreams.Container","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Datadog.DataStreams.Container.html#child_spec/1","title":"Datadog.DataStreams.Container.child_spec/1","type":"function"},{"doc":"Returns the current container id.","ref":"Datadog.DataStreams.Container.html#get/0","title":"Datadog.DataStreams.Container.get/0","type":"function"},{"doc":"Attempts to return the container id from the provided file stream. Empty on failure.","ref":"Datadog.DataStreams.Container.html#parse_container_id/1","title":"Datadog.DataStreams.Container.parse_container_id/1","type":"function"},{"doc":"Attempts to return the container id from the cgroup path ( /proc/self/cgroup ). Empty on failure.","ref":"Datadog.DataStreams.Container.html#read_container_id/0","title":"Datadog.DataStreams.Container.read_container_id/0","type":"function"},{"doc":"Attempts to return the container id from the provided file path. Empty on failure.","ref":"Datadog.DataStreams.Container.html#read_container_id/1","title":"Datadog.DataStreams.Container.read_container_id/1","type":"function"},{"doc":"Starts the agent and stores the current container id in memory.","ref":"Datadog.DataStreams.Container.html#start_link/1","title":"Datadog.DataStreams.Container.start_link/1","type":"function"},{"doc":"This module deals with storing a Datadog.DataStreams.Pathway in process, so it can be propagated later on. In Golang, this is handled via Context which is a grab bag that is passed down the whole calling stack. We don't have an equivalent in Elixir, so we instead piggy back off of OpenTelemetry.Baggage . This essentially lets us do the same thing without needing to manually pass data down the calling stack. This is optional . If you do not have Open Telemetry installed or setup, functions here will not propagate the Pathway. You will need to manually propagate it in your application.","ref":"Datadog.DataStreams.Context.html","title":"Datadog.DataStreams.Context","type":"module"},{"doc":"Removes the current existing Pathway from OpenTelemetry. Returns the value that existing in OpenTelemetry.","ref":"Datadog.DataStreams.Context.html#clear/0","title":"Datadog.DataStreams.Context.clear/0","type":"function"},{"doc":"Returns the current existing Pathway from OpenTelemetry. If there is no Pathway in the current context, nil will be returned","ref":"Datadog.DataStreams.Context.html#get/0","title":"Datadog.DataStreams.Context.get/0","type":"function"},{"doc":"Sets the given Pathway to the current Pathway in OpenTelemetry.","ref":"Datadog.DataStreams.Context.html#set/1","title":"Datadog.DataStreams.Context.set/1","type":"function"},{"doc":"Sets a checkpoint on the current existing Pathway. If one does not exist, a new Pathway is created from the edge tags and returned.","ref":"Datadog.DataStreams.Context.html#set_checkpoint/1","title":"Datadog.DataStreams.Context.set_checkpoint/1","type":"function"},{"doc":"A pathway is used to monitor how payloads are sent across different services. An example pathway would be: service A -- edge 1 --&gt; service B -- edge 2 --&gt; service C So it's a bunch of services (we also call them &quot;nodes&quot;) connected via edges. As the payload is sent around, we save the start time (start of service A), and the start time of the previous service. This allows us to measure the latency of each edge, as well as the latency from origin of any service. See the data-streams-go package for more details.","ref":"Datadog.DataStreams.Pathway.html","title":"Datadog.DataStreams.Pathway","type":"module"},{"doc":"Merges multiple pathways into one. The current implementation samples one resulting pathway. A future implementation could be more clever and actually merge the pathways.","ref":"Datadog.DataStreams.Pathway.html#merge/1","title":"Datadog.DataStreams.Pathway.merge/1","type":"function"},{"doc":"Creates a new pathway struct.","ref":"Datadog.DataStreams.Pathway.html#new_pathway/1","title":"Datadog.DataStreams.Pathway.new_pathway/1","type":"function"},{"doc":"Creates a new pathway at a given time in unix epoch nanoseconds.","ref":"Datadog.DataStreams.Pathway.html#new_pathway/2","title":"Datadog.DataStreams.Pathway.new_pathway/2","type":"function"},{"doc":"Hashes all data for a pathway. Examples iex&gt; Pathway . node_hash ( &quot;service-1&quot; , &quot;env&quot; , &quot;d:1&quot; , [ ] ) 2071821778175304604 iex&gt; # Invalid edge tag ...&gt; Pathway . node_hash ( &quot;service-1&quot; , &quot;env&quot; , &quot;d:1&quot; , [ { &quot;edge&quot; , &quot;1&quot; } ] ) 2071821778175304604 iex&gt; Pathway . node_hash ( &quot;service-1&quot; , &quot;env&quot; , &quot;d:1&quot; , [ { &quot;type&quot; , &quot;kafka&quot; } ] ) 9272613839978655432","ref":"Datadog.DataStreams.Pathway.html#node_hash/4","title":"Datadog.DataStreams.Pathway.node_hash/4","type":"function"},{"doc":"Hashes together a node and parent hash Examples iex&gt; Pathway . pathway_hash ( 0 , 0 ) 9808874869469701221 iex&gt; Pathway . pathway_hash ( 2071821778175304604 , 0 ) 17210443572488294574 iex&gt; Pathway . pathway_hash ( 0 , 2071821778175304604 ) 12425197808660046030 iex&gt; Pathway . pathway_hash ( 2071821778175304604 , 17210443572488294574 ) 2003974475228685984","ref":"Datadog.DataStreams.Pathway.html#pathway_hash/2","title":"Datadog.DataStreams.Pathway.pathway_hash/2","type":"function"},{"doc":"Sets a checkpoint on the pathway.","ref":"Datadog.DataStreams.Pathway.html#set_checkpoint/2","title":"Datadog.DataStreams.Pathway.set_checkpoint/2","type":"function"},{"doc":"Sets a checkpoint on the pathway at the given time in unix epoch nanoseconds.","ref":"Datadog.DataStreams.Pathway.html#set_checkpoint/3","title":"Datadog.DataStreams.Pathway.set_checkpoint/3","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Pathway.html#t:t/0","title":"Datadog.DataStreams.Pathway.t/0","type":"type"},{"doc":"Encoding logic for a payload. This is the top level struct we send to Datadog. It wraps all other information. These are primarily sent via MessagePack , although uses Protobuf encoded binary for latency records.","ref":"Datadog.DataStreams.Payload.html","title":"Datadog.DataStreams.Payload","type":"module"},{"doc":"Adds an aggregator bucket to the payload.","ref":"Datadog.DataStreams.Payload.html#add_bucket/2","title":"Datadog.DataStreams.Payload.add_bucket/2","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Payload.html#add_bucket/3","title":"Datadog.DataStreams.Payload.add_bucket/3","type":"function"},{"doc":"Adds a map of buckets. This is the format the the aggregator uses internally. We throw out the hash and just keep the buckets.","ref":"Datadog.DataStreams.Payload.html#add_buckets/3","title":"Datadog.DataStreams.Payload.add_buckets/3","type":"function"},{"doc":"Encodes the payload via MessagePack.","ref":"Datadog.DataStreams.Payload.html#encode/1","title":"Datadog.DataStreams.Payload.encode/1","type":"function"},{"doc":"Creates a new payload with the environment, service, and primary_tag filled in from the Datadog.DataStreams.Config module.","ref":"Datadog.DataStreams.Payload.html#new/0","title":"Datadog.DataStreams.Payload.new/0","type":"function"},{"doc":"Returns how many stats are in the payload.","ref":"Datadog.DataStreams.Payload.html#stats_count/1","title":"Datadog.DataStreams.Payload.stats_count/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Payload.html#t:t/0","title":"Datadog.DataStreams.Payload.t/0","type":"type"},{"doc":"Handles propagating Datadog.DataStreams.Pathway via encoding and adding to message headers.","ref":"Datadog.DataStreams.Propagator.html","title":"Datadog.DataStreams.Propagator","type":"module"},{"doc":"Tries to decode a value into a pathway. Examples # Verified from golang implementation iex&gt; Propagator . decode ( &lt;&lt; 174 , 208 , 17 , 141 , 62 , 199 , 215 , 238 , 224 , 159 , 240 , 170 , 211 , 97 , 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; ) % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } iex&gt; Propagator . decode ( &quot;invalid&quot; ) nil","ref":"Datadog.DataStreams.Propagator.html#decode/1","title":"Datadog.DataStreams.Propagator.decode/1","type":"function"},{"doc":"Decodes a pathway from a list or map of headers. If no matching header, or if the header is invalid, nil is returned. Examples iex&gt; Propagator . decode_header ( [ { &quot;dd-pathway-ctx-base64&quot; , &quot;rtARjT7H1+7gn/Cq02Hgn/Cq02E=&quot; } ] ) % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } iex&gt; Propagator . decode_header ( %{ &quot;dd-pathway-ctx&quot; =&gt; &lt;&lt; 174 , 208 , 17 , 141 , 62 , 199 , 215 , 238 , 224 , 159 , 240 , 170 , 211 , 97 , 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; } ) % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } iex&gt; Propagator . decode_header ( %{ &quot;content-type&quot; =&gt; &quot;application-json&quot; } ) nil","ref":"Datadog.DataStreams.Propagator.html#decode_header/1","title":"Datadog.DataStreams.Propagator.decode_header/1","type":"function"},{"doc":"Tries to decode a Base64 encoded value into a pathway. Examples # Verified from golang implementation iex&gt; Propagator . decode_str ( &quot;rtARjT7H1+7gn/Cq02Hgn/Cq02E=&quot; ) % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } iex&gt; Propagator . decode_str ( &quot;invalid&quot; ) nil","ref":"Datadog.DataStreams.Propagator.html#decode_str/1","title":"Datadog.DataStreams.Propagator.decode_str/1","type":"function"},{"doc":"Decodes a pathway binary time from zigzag encoding. Examples # Verified from golang implementation iex&gt; Propagator . decode_time ( &lt;&lt; 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; ) 1677632342000000000 iex&gt; Propagator . decode_time ( &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; ) nil","ref":"Datadog.DataStreams.Propagator.html#decode_time/1","title":"Datadog.DataStreams.Propagator.decode_time/1","type":"function"},{"doc":"Encodes a pathway to a string able to be placed in a header. Examples # Verified from golang implementation iex&gt; Propagator . encode ( % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } ) &lt;&lt; 174 , 208 , 17 , 141 , 62 , 199 , 215 , 238 , 224 , 159 , 240 , 170 , 211 , 97 , 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; # Verified from golang implementation iex&gt; Propagator . encode ( % Pathway { hash : 2003974475228685984 , pathway_start : 1677628446000000000 , edge_start : 1677628446000000000 } ) &lt;&lt; 160 , 166 , 244 , 238 , 42 , 140 , 207 , 27 , 224 , 212 , 148 , 167 , 211 , 97 , 224 , 212 , 148 , 167 , 211 , 97 &gt;&gt;","ref":"Datadog.DataStreams.Propagator.html#encode/1","title":"Datadog.DataStreams.Propagator.encode/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Propagator.html#encode_header/1","title":"Datadog.DataStreams.Propagator.encode_header/1","type":"function"},{"doc":"Encodes a pathway into a list or map of headers. Examples iex&gt; Propagator . encode_header ( [ ] , % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } ) [ { &quot;dd-pathway-ctx&quot; , &lt;&lt; 174 , 208 , 17 , 141 , 62 , 199 , 215 , 238 , 224 , 159 , 240 , 170 , 211 , 97 , 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; } ] iex&gt; Propagator . encode_header ( %{ } , % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } ) %{ &quot;dd-pathway-ctx&quot; =&gt; &lt;&lt; 174 , 208 , 17 , 141 , 62 , 199 , 215 , 238 , 224 , 159 , 240 , 170 , 211 , 97 , 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt; }","ref":"Datadog.DataStreams.Propagator.html#encode_header/2","title":"Datadog.DataStreams.Propagator.encode_header/2","type":"function"},{"doc":"Encodes a pathway to a string able to be placed in a header. Examples # Verified from golang implementation iex&gt; Propagator . encode_str ( % Pathway { hash : 17210443572488294574 , pathway_start : 1677632342000000000 , edge_start : 1677632342000000000 } ) &quot;rtARjT7H1+7gn/Cq02Hgn/Cq02E=&quot; # Verified from golang implementation iex&gt; Propagator . encode_str ( % Pathway { hash : 2003974475228685984 , pathway_start : 1677628446000000000 , edge_start : 1677628446000000000 } ) &quot;oKb07iqMzxvg1JSn02Hg1JSn02E=&quot;","ref":"Datadog.DataStreams.Propagator.html#encode_str/1","title":"Datadog.DataStreams.Propagator.encode_str/1","type":"function"},{"doc":"Encodes a pathway time using zigzag encoding. Examples # Verified from golang implementation iex&gt; Propagator . encode_time ( 1677632342000000000 ) &lt;&lt; 224 , 159 , 240 , 170 , 211 , 97 &gt;&gt;","ref":"Datadog.DataStreams.Propagator.html#encode_time/1","title":"Datadog.DataStreams.Propagator.encode_time/1","type":"function"},{"doc":"Returns the well known header key for propagating encoded pathway data. Examples iex&gt; Propagator . propagation_key ( ) &quot;dd-pathway-ctx&quot;","ref":"Datadog.DataStreams.Propagator.html#propagation_key/0","title":"Datadog.DataStreams.Propagator.propagation_key/0","type":"function"},{"doc":"Returns the well known base64 encoded header key for propagating encoded pathway data. Examples iex&gt; Propagator . propagation_key_base64 ( ) &quot;dd-pathway-ctx-base64&quot;","ref":"Datadog.DataStreams.Propagator.html#propagation_key_base64/0","title":"Datadog.DataStreams.Propagator.propagation_key_base64/0","type":"function"},{"doc":"A helper module to enumerate and filter over data stream tags.","ref":"Datadog.DataStreams.Tags.html","title":"Datadog.DataStreams.Tags","type":"module"},{"doc":"Tags a list of tags and converts them to a list of binary tags. This is the format that Datadog expects. This also orders the tags to ensure it matches other languages when hashing. Examples iex&gt; Tags . encode ( [ { &quot;tag_two&quot; , &quot;value_two&quot; } , { &quot;tag&quot; , &quot;value&quot; } ] ) [ &quot;tag:value&quot; , &quot;tag_two:value_two&quot; ]","ref":"Datadog.DataStreams.Tags.html#encode/1","title":"Datadog.DataStreams.Tags.encode/1","type":"function"},{"doc":"Filters a list of tags to a known list of allowed tags. This varies based on the context of where it is used. Examples iex&gt; [ { &quot;key&quot; , &quot;value&quot; } , { &quot;topic&quot; , &quot;one&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] ...&gt; |&gt; Tags . filter ( :hash ) [ { &quot;topic&quot; , &quot;one&quot; } ]","ref":"Datadog.DataStreams.Tags.html#filter/2","title":"Datadog.DataStreams.Tags.filter/2","type":"function"},{"doc":"Parses any type of tag input and normalizes it to our internal t:t type. Note this will also filter out invalid data like nil values. Examples iex&gt; [ { :key , &quot;value&quot; } , { :key_two , &quot;value_two&quot; } ] ...&gt; |&gt; Tags . parse ( ) [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] iex&gt; [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] ...&gt; |&gt; Tags . parse ( ) [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] iex&gt; %{ key : &quot;value&quot; , key_two : &quot;value_two&quot; } ...&gt; |&gt; Tags . parse ( ) [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] iex&gt; %{ &quot;key&quot; =&gt; &quot;value&quot; , &quot;key_two&quot; =&gt; &quot;value_two&quot; } ...&gt; |&gt; Tags . parse ( ) [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] iex&gt; [ &quot;key:value&quot; , &quot;key_two:value_two&quot; ] ...&gt; |&gt; Tags . parse ( ) [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] iex&gt; [ { &quot;key&quot; , nil } , { &quot;key_two&quot; , &quot;value_two&quot; } ] ...&gt; |&gt; Tags . parse ( ) [ { &quot;key_two&quot; , &quot;value_two&quot; } ]","ref":"Datadog.DataStreams.Tags.html#parse/1","title":"Datadog.DataStreams.Tags.parse/1","type":"function"},{"doc":"An encoded list of tags. This is the format that Datadog expects when we send data. It's a list of key value binary joined via : . For example: &quot;partition:1&quot;, &quot;group:some-consumer-group&quot;, &quot;topic:a-topic&quot;.","ref":"Datadog.DataStreams.Tags.html#t:encoded/0","title":"Datadog.DataStreams.Tags.encoded/0","type":"type"},{"doc":"All allowed tag input types. This could be any one of the following examples: [ { :key , &quot;value&quot; } , { :key_two , &quot;value_two&quot; } ] [ { &quot;key&quot; , &quot;value&quot; } , { &quot;key_two&quot; , &quot;value_two&quot; } ] %{ key : &quot;value&quot; , key_two : &quot;value_two&quot; } %{ &quot;key&quot; =&gt; &quot;value&quot; , &quot;key_two&quot; =&gt; &quot;value_two&quot; } [ &quot;key:value&quot; , &quot;key_two:value_two&quot; ]","ref":"Datadog.DataStreams.Tags.html#t:input/0","title":"Datadog.DataStreams.Tags.input/0","type":"type"},{"doc":"The type all tags are internally mapped to.","ref":"Datadog.DataStreams.Tags.html#t:t/0","title":"Datadog.DataStreams.Tags.t/0","type":"type"},{"doc":"An HTTP client for Datadog data streams reporting. It uses the Finch library for requests.","ref":"Datadog.DataStreams.Transport.html","title":"Datadog.DataStreams.Transport","type":"module"},{"doc":"Sends a MessagePack-ed binary to the Datadog agent. Ensuring it is acknowledged by response before returning :ok .","ref":"Datadog.DataStreams.Transport.html#send_pipeline_stats/1","title":"Datadog.DataStreams.Transport.send_pipeline_stats/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Transport.html#t:response/0","title":"Datadog.DataStreams.Transport.response/0","type":"type"},{"doc":"A minimal implementation of the distributed quantile sketch (DDSketch) algorithm as implemented in the sketches-go library. For more information, please read the sketches-go README . This is a minimal implementation needed to support DataDog data streams. Some initial work was taken from the great dog_sketch Elixir library . This library includes some DataDog specific work like Protobuf encoding. It's worth mentioning that some of this code looks out of place in Elixir. That is because it's pulled directly from the sketches-go library and kept similar for ease of debugging and backporting fixes.","ref":"Datadog.Sketch.html","title":"Datadog.Sketch","type":"module"},{"doc":"Adds a value to the sketch. Examples iex&gt; % Sketch { } = Sketch . add ( Sketch . new_default ( ) , 4293.7 ) iex&gt; % Sketch { } = Sketch . add ( Sketch . new_default ( ) , - 4592.3 ) iex&gt; % Sketch { } = Sketch . add ( Sketch . new_default ( ) , 0.0 )","ref":"Datadog.Sketch.html#add/2","title":"Datadog.Sketch.add/2","type":"function"},{"doc":"Adds a bin type to the sketch. Examples iex&gt; % Sketch { } = Sketch . add_bin ( Sketch . new_default ( ) , %{ index : 100 , count : 13.13 } )","ref":"Datadog.Sketch.html#add_bin/2","title":"Datadog.Sketch.add_bin/2","type":"function"},{"doc":"Adds multiple bin types to the sketch. Examples iex&gt; % Sketch { } = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 100 , count : 13.13 } , ...&gt; %{ index : 20 , count : 2342.4 } ...&gt; ] )","ref":"Datadog.Sketch.html#add_bins/2","title":"Datadog.Sketch.add_bins/2","type":"function"},{"doc":"Adds a value to the sketch with a float count. Examples iex&gt; % Sketch { } = Sketch . add_with_count ( Sketch . new_default ( ) , 4293.7 , 294 ) iex&gt; % Sketch { } = Sketch . add_with_count ( Sketch . new_default ( ) , - 4592.3 , 23 ) iex&gt; % Sketch { } = Sketch . add_with_count ( Sketch . new_default ( ) , 0.0 , 10 )","ref":"Datadog.Sketch.html#add_with_count/3","title":"Datadog.Sketch.add_with_count/3","type":"function"},{"doc":"Returns true if no value has been added to this sketch. Examples iex&gt; Sketch . empty? ( Sketch . new_default ( ) ) true iex&gt; Sketch . new_default ( ) ...&gt; |&gt; Sketch . add_with_count ( 42 , 482.23 ) ...&gt; |&gt; Sketch . empty? ( ) false iex&gt; Sketch . new_default ( ) ...&gt; |&gt; Sketch . add_with_count ( - 75 , 157 ) ...&gt; |&gt; Sketch . empty? ( ) false","ref":"Datadog.Sketch.html#empty?/1","title":"Datadog.Sketch.empty?/1","type":"function"},{"doc":"Return the total number of values that have been added to this sketch. Examples iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 423.43 } , ...&gt; %{ index : 244 , count : 1238.123 } , ...&gt; %{ index : 124 , count : 2184.124 } ...&gt; ] ) ...&gt; Sketch . get_count ( sketch ) 3845.6769999999997","ref":"Datadog.Sketch.html#get_count/1","title":"Datadog.Sketch.get_count/1","type":"function"},{"doc":"Returns the maximum value that has been added to this sketch. Examples iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 34 } , ...&gt; %{ index : - 423 , count : 571 } , ...&gt; %{ index : 0 , count : 27.1 } ...&gt; ] ) ...&gt; Sketch . get_max_value ( sketch ) 12.061674179039226","ref":"Datadog.Sketch.html#get_max_value/1","title":"Datadog.Sketch.get_max_value/1","type":"function"},{"doc":"Returns the minimum value that has been added to this sketch. Examples iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 34 } , ...&gt; %{ index : - 423 , count : 571 } , ...&gt; %{ index : 0 , count : 27.1 } ...&gt; ] ) ...&gt; Sketch . get_min_value ( sketch ) - 424.1773628048435","ref":"Datadog.Sketch.html#get_min_value/1","title":"Datadog.Sketch.get_min_value/1","type":"function"},{"doc":"Returns an approximation of the sum of the values that have been added to the sketch. If the values that have been added to the sketch all have the same sign, the approximation error has the relative accuracy guarantees of the mapping used for this sketch. Examples # Verified with golang implementation (within accuracy) iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 34.0 } , ...&gt; %{ index : - 24 , count : 84.0 } , ...&gt; %{ index : 0 , count : 2.4 } ...&gt; ] ) ...&gt; Sketch . get_sum ( sketch ) - 1589.8430984284082","ref":"Datadog.Sketch.html#get_sum/1","title":"Datadog.Sketch.get_sum/1","type":"function"},{"doc":"Return the value at the specified quantile. Examples # Validated with golang implementation iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 423.43 } , ...&gt; %{ index : 244 , count : 1238.123 } , ...&gt; %{ index : 124 , count : 2184.124 } ...&gt; ] ) ...&gt; Sketch . get_value_at_quantile ( sketch , 0.4 ) 125.2248607394614","ref":"Datadog.Sketch.html#get_value_at_quantile/2","title":"Datadog.Sketch.get_value_at_quantile/2","type":"function"},{"doc":"Return the values at the respective specified quantiles. Examples # Validated with golang implementation (within accuracy) iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 423.43 } , ...&gt; %{ index : 244 , count : 1238.123 } , ...&gt; %{ index : 124 , count : 2184.124 } ...&gt; ] ) ...&gt; Sketch . get_values_at_quantiles ( sketch , [ 0.1 , 0.25 , 0.5 ] ) [ 12.061674179039226 , 125.2248607394614 , 125.2248607394614 ]","ref":"Datadog.Sketch.html#get_values_at_quantiles/2","title":"Datadog.Sketch.get_values_at_quantiles/2","type":"function"},{"doc":"Returns the number of zero values that have been added to this sketch. Examples iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : 12 , count : 34 } , ...&gt; %{ index : - 423 , count : 571 } , ...&gt; %{ index : 0 , count : 27.1 } ...&gt; ] ) ...&gt; Sketch . get_zero_count ( sketch ) 27.1","ref":"Datadog.Sketch.html#get_zero_count/1","title":"Datadog.Sketch.get_zero_count/1","type":"function"},{"doc":"Creates a new Sketch. Examples iex&gt; index_mapping = Sketch.IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; store = Sketch.Store.Dense . new ( ) ...&gt; % Sketch { } = Sketch . new ( index_mapping , store )","ref":"Datadog.Sketch.html#new/2","title":"Datadog.Sketch.new/2","type":"function"},{"doc":"Creates a new Sketch with separate stores for positive and negative values. Examples iex&gt; index_mapping = Sketch.IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; positive_store = Sketch.Store.Dense . new ( ) ...&gt; negative_store = Sketch.Store.Dense . new ( ) ...&gt; % Sketch { } = Sketch . new ( index_mapping , positive_store , negative_store )","ref":"Datadog.Sketch.html#new/3","title":"Datadog.Sketch.new/3","type":"function"},{"doc":"Creates a new Sketch with the default index mapping and store values. This equates to using the IndexMapping.Logarithmic module with a 0.01 relative accuracy and the Store.Dense store. Examples iex&gt; % Sketch { } = Sketch . new_default ( )","ref":"Datadog.Sketch.html#new_default/0","title":"Datadog.Sketch.new_default/0","type":"function"},{"doc":"Reweight multiples all values from the sketch by weight , but keeps the same global distribution. weight has to be strictly greater than zero. Examples # Verified with golang implementation (within accuracy) iex&gt; sketch = Sketch . add_bins ( Sketch . new_default ( ) , [ ...&gt; %{ index : - 7 , count : 10.0 } , ...&gt; %{ index : 24 , count : 20.0 } , ...&gt; %{ index : 3 , count : 30.0 } ...&gt; ] ) ...&gt; sketch = Sketch . reweight ( sketch , 2.5 ) ...&gt; Sketch . get_sum ( sketch ) 1237.7881696246109","ref":"Datadog.Sketch.html#reweight/2","title":"Datadog.Sketch.reweight/2","type":"function"},{"doc":"Returns a Protobuf-able struct for the sketch. Used for sending data to Datadog. Examples iex&gt; % Sketch.Protobuf.DDSketch { } = Sketch . to_proto ( Sketch . new_default ( ) )","ref":"Datadog.Sketch.html#to_proto/1","title":"Datadog.Sketch.to_proto/1","type":"function"},{"doc":"","ref":"Datadog.Sketch.html#t:t/0","title":"Datadog.Sketch.t/0","type":"type"},{"doc":"Basic module for handling various index mapping algorithms. All functions in this module proxy to the respective index mapping implementation module.","ref":"Datadog.Sketch.IndexMapping.html","title":"Datadog.Sketch.IndexMapping","type":"behaviour"},{"doc":"Checks if an index mapping matches another index mapping. iex&gt; implementation_one = IndexMapping.Logarithmic . new ( 0.0000000000001 ) ...&gt; implementation_two = IndexMapping.Logarithmic . new ( 0.0000000000002 ) ...&gt; IndexMapping . equals ( implementation_one , implementation_two ) true iex&gt; implementation_one = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; implementation_two = IndexMapping.Logarithmic . new ( 0.00001 ) ...&gt; IndexMapping . equals ( implementation_one , implementation_two ) false","ref":"Datadog.Sketch.IndexMapping.html#equals/2","title":"Datadog.Sketch.IndexMapping.equals/2","type":"function"},{"doc":"Checks if an index mapping matches another index mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:equals/2","title":"Datadog.Sketch.IndexMapping.equals/2","type":"callback"},{"doc":"Returns value after mapping. iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . index ( index_mapping , 115 ) 237 iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.001 ) ...&gt; IndexMapping . index ( index_mapping , 12345678901234567890 ) 21979","ref":"Datadog.Sketch.IndexMapping.html#index/2","title":"Datadog.Sketch.IndexMapping.index/2","type":"function"},{"doc":"Returns value after mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:index/2","title":"Datadog.Sketch.IndexMapping.index/2","type":"callback"},{"doc":"Returns the lower bound the mapping can contain. iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . lower_bound ( index_mapping , 0 ) 1.0 iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . lower_bound ( index_mapping , 10 ) 1.2214109013609646","ref":"Datadog.Sketch.IndexMapping.html#lower_bound/2","title":"Datadog.Sketch.IndexMapping.lower_bound/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain.","ref":"Datadog.Sketch.IndexMapping.html#c:lower_bound/2","title":"Datadog.Sketch.IndexMapping.lower_bound/2","type":"callback"},{"doc":"Returns the lower bound the mapping can contain. iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . relative_accuracy ( index_mapping ) 0.009999999999999898","ref":"Datadog.Sketch.IndexMapping.html#relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.relative_accuracy/1","type":"function"},{"doc":"Returns the relative accuracy of the mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.relative_accuracy/1","type":"callback"},{"doc":"Returns a Protobuf-able struct for the index mapping. Used for sending data to Datadog. iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . to_proto ( index_mapping ) % Datadog.Sketch.Protobuf.IndexMapping { gamma : 1.02020202020202 , interpolation : :NONE }","ref":"Datadog.Sketch.IndexMapping.html#to_proto/1","title":"Datadog.Sketch.IndexMapping.to_proto/1","type":"function"},{"doc":"Returns a Protobuf-able struct for the index mapping. Used for sending data to Datadog.","ref":"Datadog.Sketch.IndexMapping.html#c:to_proto/1","title":"Datadog.Sketch.IndexMapping.to_proto/1","type":"callback"},{"doc":"Takes a mapped value and returns the original value within the set accuracy. iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.01 ) ...&gt; IndexMapping . value ( index_mapping , 237 ) 115.59680764552533 iex&gt; index_mapping = IndexMapping.Logarithmic . new ( 0.001 ) ...&gt; IndexMapping . value ( index_mapping , 21979 ) 1.23355147396003e19","ref":"Datadog.Sketch.IndexMapping.html#value/2","title":"Datadog.Sketch.IndexMapping.value/2","type":"function"},{"doc":"Takes a mapped value and returns the original value within the set accuracy.","ref":"Datadog.Sketch.IndexMapping.html#c:value/2","title":"Datadog.Sketch.IndexMapping.value/2","type":"callback"},{"doc":"Checks if the two values are within the tolerance given. Examples iex&gt; IndexMapping . within_tolerance ( 90 , 134 , 50 ) true iex&gt; IndexMapping . within_tolerance ( 0.00128 , 0.00864 , 0.01 ) false","ref":"Datadog.Sketch.IndexMapping.html#within_tolerance/3","title":"Datadog.Sketch.IndexMapping.within_tolerance/3","type":"function"},{"doc":"A general struct that has index mapping data. Every index mapping implementation must contain this data.","ref":"Datadog.Sketch.IndexMapping.html#t:t/0","title":"Datadog.Sketch.IndexMapping.t/0","type":"type"},{"doc":"LogarithmicMapping is an IndexMapping that is memory-optimal, that is to say that given a targeted relative accuracy, it requires the least number of indices to cover a given range of values. This is done by logarithmically mapping floating-point values to integers. Note, since Erlang (and therefor Elixir) do math pretty differently than golang, this module does not contain a min or max indexable value. This follows in line with the Erlang philosophy of &quot;let it crash&quot; and simplifies our logic handling.","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html","title":"Datadog.Sketch.IndexMapping.Logarithmic","type":"module"},{"doc":"Checks if an index mapping matches another index mapping. iex&gt; implementation_one = Logarithmic . new ( 0.0000000000001 ) ...&gt; implementation_two = Logarithmic . new ( 0.0000000000002 ) ...&gt; Logarithmic . equals ( implementation_one , implementation_two ) true iex&gt; implementation_one = Logarithmic . new ( 0.01 ) ...&gt; implementation_two = Logarithmic . new ( 0.00001 ) ...&gt; Logarithmic . equals ( implementation_one , implementation_two ) false","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#equals/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.equals/2","type":"function"},{"doc":"Returns value after mapping via logarithmic equation. iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . index ( index_mapping , 115 ) 237 iex&gt; index_mapping = Logarithmic . new ( 0.001 ) ...&gt; Logarithmic . index ( index_mapping , 12345678901234567890 ) 21979","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#index/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.index/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain. iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . lower_bound ( index_mapping , 0 ) 1.0 iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . lower_bound ( index_mapping , 10 ) 1.2214109013609646","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#lower_bound/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.lower_bound/2","type":"function"},{"doc":"Creates a new Logarithmic index mapping with the given accuracy. Examples iex&gt; Logarithmic . new ( - 1 ) ** (ArgumentError) The relative accuracy must be between 0, and 1. iex&gt; Logarithmic . new ( 0.01 ) % Logarithmic { gamma : 1.02020202020202 , multiplier : 49.99833328888678 } iex&gt; Logarithmic . new ( 0.0001 ) % Logarithmic { gamma : 1.0002000200020003 , multiplier : 4999.999983331928 }","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#new/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.new/1","type":"function"},{"doc":"Creates a new Logarithmic index mapping with the given gamma and index offset. iex&gt; Logarithmic . new ( 0.5 , 0.0 ) ** (ArgumentError) Gamma must be greater than 1. iex&gt; Logarithmic . new ( 1.0002000200020003 , 0.0 ) % Logarithmic { gamma : 1.0002000200020003 , multiplier : 4999.999983331928 } iex&gt; Logarithmic . new ( 1.0002000200020003 , 1.5 ) % Logarithmic { gamma : 1.0002000200020003 , index_offset : 1.5 , multiplier : 4999.999983331928 }","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#new/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.new/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain. iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . relative_accuracy ( index_mapping ) 0.009999999999999898","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.relative_accuracy/1","type":"function"},{"doc":"Returns a Protobuf-able struct for the index mapping., Used for sending data to Datadog. iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . to_proto ( index_mapping ) % Datadog.Sketch.Protobuf.IndexMapping { gamma : 1.02020202020202 , interpolation : :NONE }","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#to_proto/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.to_proto/1","type":"function"},{"doc":"Takes a mapped value and returns the original value within the set accuracy. iex&gt; index_mapping = Logarithmic . new ( 0.01 ) ...&gt; Logarithmic . value ( index_mapping , 237 ) 115.59680764552533 iex&gt; index_mapping = Logarithmic . new ( 0.001 ) ...&gt; Logarithmic . value ( index_mapping , 21979 ) 1.23355147396003e19","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#value/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.value/2","type":"function"},{"doc":"","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#t:t/0","title":"Datadog.Sketch.IndexMapping.Logarithmic.t/0","type":"type"},{"doc":"Different stores use different data structures and techniques to store data. Each with unique trade offs and memory usage. All functions in this module proxy to the respective index mapping implementation module. All stores implement the Enumerable protocol to iterate over all stored data easily.","ref":"Datadog.Sketch.Store.html","title":"Datadog.Sketch.Store","type":"behaviour"},{"doc":"Adds a number to the store. Examples iex&gt; % Store.Dense { } = Store . add ( Store.Dense . new ( ) , 100 )","ref":"Datadog.Sketch.Store.html#add/2","title":"Datadog.Sketch.Store.add/2","type":"function"},{"doc":"Adds a number to the store.","ref":"Datadog.Sketch.Store.html#c:add/2","title":"Datadog.Sketch.Store.add/2","type":"callback"},{"doc":"Adds a bin type to the store. Examples iex&gt; % Store.Dense { } = Store . add_bin ( Store.Dense . new ( ) , %{ index : 100 , count : 13.13 } )","ref":"Datadog.Sketch.Store.html#add_bin/2","title":"Datadog.Sketch.Store.add_bin/2","type":"function"},{"doc":"Adds a bin type to the store.","ref":"Datadog.Sketch.Store.html#c:add_bin/2","title":"Datadog.Sketch.Store.add_bin/2","type":"callback"},{"doc":"Adds multiple bin types to the store. Examples iex&gt; % Store.Dense { } = Store . add_bins ( Store.Dense . new ( ) , [ ...&gt; %{ index : 100 , count : 13.13 } , ...&gt; %{ index : 20 , count : 2342.4 } ...&gt; ] )","ref":"Datadog.Sketch.Store.html#add_bins/2","title":"Datadog.Sketch.Store.add_bins/2","type":"function"},{"doc":"Adds multiple bin types to the store.","ref":"Datadog.Sketch.Store.html#c:add_bins/2","title":"Datadog.Sketch.Store.add_bins/2","type":"callback"},{"doc":"Adds a number to the store count number of times. Examples iex&gt; % Store.Dense { } = Store . add_with_count ( Store.Dense . new ( ) , 100 , 13.13 ) iex&gt; % Store.Dense { } = Store . add_with_count ( Store.Dense . new ( ) , 987 , 8.3e12 )","ref":"Datadog.Sketch.Store.html#add_with_count/3","title":"Datadog.Sketch.Store.add_with_count/3","type":"function"},{"doc":"Adds a number to the store count number of times.","ref":"Datadog.Sketch.Store.html#c:add_with_count/3","title":"Datadog.Sketch.Store.add_with_count/3","type":"callback"},{"doc":"Checks if the store has any information in it. Examples iex&gt; store = Store.Dense . new ( ) ...&gt; Store . empty? ( store ) true iex&gt; store = Store . add ( Store.Dense . new ( ) , 754 ) ...&gt; Store . empty? ( store ) false","ref":"Datadog.Sketch.Store.html#empty?/1","title":"Datadog.Sketch.Store.empty?/1","type":"function"},{"doc":"Checks if the store has any information in it.","ref":"Datadog.Sketch.Store.html#c:empty?/1","title":"Datadog.Sketch.Store.empty?/1","type":"callback"},{"doc":"Return the key for the value at rank. Examples iex&gt; store = Store . add ( Store.Dense . new ( ) , 128 ) ...&gt; Store . key_at_rank ( store , 0.0 ) 128","ref":"Datadog.Sketch.Store.html#key_at_rank/2","title":"Datadog.Sketch.Store.key_at_rank/2","type":"function"},{"doc":"Return the key for the value at rank.","ref":"Datadog.Sketch.Store.html#c:key_at_rank/2","title":"Datadog.Sketch.Store.key_at_rank/2","type":"callback"},{"doc":"Returns the maximum index of the store. Examples iex&gt; store = Store.Dense . new ( ) ...&gt; Store . max_index ( store ) 0","ref":"Datadog.Sketch.Store.html#max_index/1","title":"Datadog.Sketch.Store.max_index/1","type":"function"},{"doc":"Returns the maximum index of the store.","ref":"Datadog.Sketch.Store.html#c:max_index/1","title":"Datadog.Sketch.Store.max_index/1","type":"callback"},{"doc":"Returns the minimum index of the store. Examples iex&gt; store = Store.Dense . new ( ) ...&gt; Store . min_index ( store ) 0","ref":"Datadog.Sketch.Store.html#min_index/1","title":"Datadog.Sketch.Store.min_index/1","type":"function"},{"doc":"Returns the minimum index of the store.","ref":"Datadog.Sketch.Store.html#c:min_index/1","title":"Datadog.Sketch.Store.min_index/1","type":"callback"},{"doc":"Maps over all values and multiplies by the given weight. Examples iex&gt; store = Store . add_bins ( Store.Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 10.0 } , ...&gt; %{ index : 2 , count : 20.0 } , ...&gt; %{ index : 6 , count : 30.0 } ...&gt; ] ) ...&gt; store = Store . reweight ( store , 2 ) ...&gt; Store . total_count ( store ) 120.0","ref":"Datadog.Sketch.Store.html#reweight/2","title":"Datadog.Sketch.Store.reweight/2","type":"function"},{"doc":"Maps over all values and multiplies by the given weight.","ref":"Datadog.Sketch.Store.html#c:reweight/2","title":"Datadog.Sketch.Store.reweight/2","type":"callback"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to Datadog. Examples iex&gt; % Datadog.Sketch.Protobuf.Store { } = Store . to_proto ( Store.Dense . new ( ) )","ref":"Datadog.Sketch.Store.html#to_proto/1","title":"Datadog.Sketch.Store.to_proto/1","type":"function"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to Datadog.","ref":"Datadog.Sketch.Store.html#c:to_proto/1","title":"Datadog.Sketch.Store.to_proto/1","type":"callback"},{"doc":"Returns the total amount of counts stored. Examples iex&gt; store = Store.Dense . new ( ) ...&gt; Store . total_count ( store ) 0.0 iex&gt; store = Store . add_with_count ( Store.Dense . new ( ) , 754 , 42.42 ) ...&gt; Store . total_count ( store ) 42.42","ref":"Datadog.Sketch.Store.html#total_count/1","title":"Datadog.Sketch.Store.total_count/1","type":"function"},{"doc":"Returns the total amount of counts stored.","ref":"Datadog.Sketch.Store.html#c:total_count/1","title":"Datadog.Sketch.Store.total_count/1","type":"callback"},{"doc":"A concise way to specify an index and count in one arg.","ref":"Datadog.Sketch.Store.html#t:bin/0","title":"Datadog.Sketch.Store.bin/0","type":"type"},{"doc":"A basic struct. This is used to keep tight type definitions. Each store implements this struct differently.","ref":"Datadog.Sketch.Store.html#t:t/0","title":"Datadog.Sketch.Store.t/0","type":"type"},{"doc":"The Dense store is a dynamically growing contiguous (non-sparse) store. The number of bins are bound only by the size of the :array that can be allocated.","ref":"Datadog.Sketch.Store.Dense.html","title":"Datadog.Sketch.Store.Dense","type":"module"},{"doc":"Adds a number to the store. Examples iex&gt; % Dense { } = Dense . add ( Dense . new ( ) , 100 )","ref":"Datadog.Sketch.Store.Dense.html#add/2","title":"Datadog.Sketch.Store.Dense.add/2","type":"function"},{"doc":"Adds a bin type to the store. Examples iex&gt; % Dense { } = Dense . add_bin ( Dense . new ( ) , %{ index : 100 , count : 13.13 } )","ref":"Datadog.Sketch.Store.Dense.html#add_bin/2","title":"Datadog.Sketch.Store.Dense.add_bin/2","type":"function"},{"doc":"Adds multiple bin types to the store. Examples iex&gt; % Dense { } = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 100 , count : 13.13 } , ...&gt; %{ index : 20 , count : 2342.4 } ...&gt; ] )","ref":"Datadog.Sketch.Store.Dense.html#add_bins/2","title":"Datadog.Sketch.Store.Dense.add_bins/2","type":"function"},{"doc":"Adds a number to the store count number of times. Examples iex&gt; % Dense { } = Dense . add_with_count ( Dense . new ( ) , 100 , 13.13 ) iex&gt; % Dense { } = Dense . add_with_count ( Dense . new ( ) , 987 , 8.3e12 )","ref":"Datadog.Sketch.Store.Dense.html#add_with_count/3","title":"Datadog.Sketch.Store.Dense.add_with_count/3","type":"function"},{"doc":"Checks if the store has any information in it. Examples iex&gt; store = Dense . new ( ) ...&gt; Dense . empty? ( store ) true iex&gt; store = Dense . add ( Dense . new ( ) , 754 ) ...&gt; Dense . empty? ( store ) false","ref":"Datadog.Sketch.Store.Dense.html#empty?/1","title":"Datadog.Sketch.Store.Dense.empty?/1","type":"function"},{"doc":"Return the key for the value at rank. Examples # Matches the golang implementation when no data is present. iex&gt; store = Dense . new ( ) ...&gt; Dense . key_at_rank ( store , 0.0 ) - 134_217_729 # Verified matching to golang implementation iex&gt; store = Dense . add ( Dense . new ( ) , 128 ) ...&gt; Dense . key_at_rank ( store , 0.0 ) 128 # Verified matching to golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 12 , count : 423.43 } , ...&gt; %{ index : 244 , count : 1238.123 } , ...&gt; %{ index : 124 , count : 2184.124 } ...&gt; ] ) ...&gt; Dense . key_at_rank ( store , 64.0 ) 12","ref":"Datadog.Sketch.Store.Dense.html#key_at_rank/2","title":"Datadog.Sketch.Store.Dense.key_at_rank/2","type":"function"},{"doc":"Returns the maximum index of the store. Examples iex&gt; store = Dense . new ( ) ...&gt; Dense . max_index ( store ) 0 iex&gt; store = Dense . add ( Dense . new ( ) , 128 ) ...&gt; Dense . max_index ( store ) 128 # Verified against golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 12.48 } , ...&gt; %{ index : 65 , count : 12.48 } , ...&gt; %{ index : 37 , count : 847.4 } ...&gt; ] ) ...&gt; Dense . max_index ( store ) 65","ref":"Datadog.Sketch.Store.Dense.html#max_index/1","title":"Datadog.Sketch.Store.Dense.max_index/1","type":"function"},{"doc":"Returns the minimum index of the store. Examples iex&gt; store = Dense . new ( ) ...&gt; Dense . min_index ( store ) 0 # Verified against golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 12.48 } , ...&gt; %{ index : 65 , count : 12.48 } , ...&gt; %{ index : 37 , count : 847.4 } ...&gt; ] ) ...&gt; Dense . min_index ( store ) 4","ref":"Datadog.Sketch.Store.Dense.html#min_index/1","title":"Datadog.Sketch.Store.Dense.min_index/1","type":"function"},{"doc":"Creates a new dense store. Examples iex&gt; Dense . new ( ) % Dense { }","ref":"Datadog.Sketch.Store.Dense.html#new/0","title":"Datadog.Sketch.Store.Dense.new/0","type":"function"},{"doc":"Maps over all values and multiplies by the given weight. Examples # Verified matching to golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 10.0 } , ...&gt; %{ index : 2 , count : 20.0 } , ...&gt; %{ index : 6 , count : 30.0 } ...&gt; ] ) ...&gt; store = Dense . reweight ( store , 2 ) ...&gt; Dense . total_count ( store ) 120.0","ref":"Datadog.Sketch.Store.Dense.html#reweight/2","title":"Datadog.Sketch.Store.Dense.reweight/2","type":"function"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to Datadog. Examples # Verified matching to golang implementation iex&gt; store = Dense . add ( Dense . new ( ) , 4 ) ...&gt; Dense . to_proto ( store ) % Datadog.Sketch.Protobuf.Store { binCounts : [ ] , contiguousBinCounts : [ 1.0 ] , contiguousBinIndexOffset : 4 } # Verified matching to golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 12.48 } , ...&gt; %{ index : 65 , count : 12.48 } , ...&gt; %{ index : 37 , count : 847.4 } ...&gt; ] ) ...&gt; Dense . to_proto ( store ) % Datadog.Sketch.Protobuf.Store { binCounts : [ ] , contiguousBinCounts : [ 12.48 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 847.4 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 12.48 ] , contiguousBinIndexOffset : 4 }","ref":"Datadog.Sketch.Store.Dense.html#to_proto/1","title":"Datadog.Sketch.Store.Dense.to_proto/1","type":"function"},{"doc":"Returns the total amount of counts stored. Examples iex&gt; store = Dense . new ( ) ...&gt; Dense . total_count ( store ) 0.0 iex&gt; store = Dense . add ( Dense . new ( ) , 754 ) ...&gt; Dense . total_count ( store ) 1.0 iex&gt; store = Dense . add_with_count ( Dense . new ( ) , 754 , 42.42 ) ...&gt; Dense . total_count ( store ) 42.42 # Verified against golang implementation iex&gt; store = Dense . add_bins ( Dense . new ( ) , [ ...&gt; %{ index : 4 , count : 12.48 } , ...&gt; %{ index : 65 , count : 12.48 } , ...&gt; %{ index : 37 , count : 847.4 } ...&gt; ] ) ...&gt; Dense . total_count ( store ) 872.36","ref":"Datadog.Sketch.Store.Dense.html#total_count/1","title":"Datadog.Sketch.Store.Dense.total_count/1","type":"function"},{"doc":"","ref":"Datadog.Sketch.Store.Dense.html#t:t/0","title":"Datadog.Sketch.Store.Dense.t/0","type":"type"},{"doc":"This is a port of the data-streams-go library to Elixir.","ref":"readme.html","title":"Data Streams Ex","type":"extras"},{"doc":"This product is meant to measure end to end latency in async pipelines. It's in an Alpha phase. We currently support instrumentations of async pipelines using Kafka. Integrations with other systems will follow soon.","ref":"readme.html#introduction","title":"Data Streams Ex - Introduction","type":"extras"},{"doc":"Data stream: A set of services connected together via queues Pathway: A single branch of connected services Queue: A connection between two services Edge latency: Latency of a queue between two services Latency from origin: Latency from the first tracked service, down to the current service Checkpoint: records at what time a specific operation on a payload occurred (eg: The payload was sent to Kafka). The product can then measure latency between checkpoints. The product can measure edge latency, and latency from origin, for a set of checkpoints connected together via queues. To do so, we propagate timestamps, and a hash of the path that messages took with the payload.","ref":"readme.html#glossary","title":"Data Streams Ex - Glossary","type":"extras"},{"doc":"Just add data_streams to your mix.exs file like so: def deps do [ { :data_streams , &quot;~&gt; 1.1.2&quot; } ] end Documentation is automatically generated and published to HexDocs .","ref":"readme.html#installation","title":"Data Streams Ex - Installation","type":"extras"},{"doc":"Prerequisites Datadog Agent 7.34.0+ latest version of the data streams library You will need to configure the pipeline with the trace agent URL and enable it to start on application start. This can be done via your config/ files: config :data_streams , :pipeline , enabled? : true , host : &quot;localhost&quot; , port : 8126 The host and port should point to your Datadog agent. The instrumentation relies on creating checkpoints at various points in your data stream services with specific tags, recording the pathway that messages take along the way. For a complete picture of your services, you will also need to configure some metadata about the current service running. You can do this in your config/ files as well: config :data_streams , :metadata , service : &quot;my-service&quot; , env : &quot;production&quot; , primary_tag : &quot;datacenter:d1&quot; # You can leave this blank if you don&#39;t have a primary tag. We recommend you keep these tags matching all your other instrumentation, like Open Telemetry and :telemetry , to ensure Datadog can aggregate data accurately. Integrations This library contains integration modules to help integrate with various async data pipelines. See one of these modules for usage details. Datadog.DataStreams.Integrations.Kafka","ref":"readme.html#elixir-instrumentation","title":"Data Streams Ex - Elixir instrumentation","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Bug Fixes defensive code around container logic ( #16 ) ( 97d770b )","ref":"changelog.html#1-1-2-2023-06-05","title":"Changelog - 1.1.2 (2023-06-05)","type":"extras"},{"doc":"Bug Fixes set span attribute to pathway hash ( #14 ) ( 94ca3fd )","ref":"changelog.html#1-1-1-2023-06-05","title":"Changelog - 1.1.1 (2023-06-05)","type":"extras"},{"doc":"Features add container id to transport headers ( #12 ) ( 4d58826 )","ref":"changelog.html#1-1-0-2023-05-23","title":"Changelog - 1.1.0 (2023-05-23)","type":"extras"},{"doc":"⚠ BREAKING CHANGES Package and application configuration is now under data_streams instead of dd_data_streams Features add basic implementation of ddsketch ( #1 ) ( 125b5ed ) add basic kafka tracking support with data streams ( #3 ) ( bfc6a0b ) add LICENSE file ( #11 ) ( 6c5668f ) link open telemetry span to current pathway context ( #5 ) ( e0ed9b2 ) rename dd_data_streams to data_streams ( #9 ) ( a0d1742 ) Bug Fixes add case for error http status ( #8 ) ( ef4a95d ) ci: update PR title regex check ( 64ef99f ) dialyzer warnings for kafka integration map ( 18bf936 ) filter out nil values from kafka integration tags ( b33926f ) update kafka integration to not set context on produce ( #7 ) ( 6807b6d ) update otel resource service configuration ( adb9890 ) update tag logic to be more consistant ( #4 ) ( 48d13df )","ref":"changelog.html#1-0-0-2023-04-06","title":"Changelog - 1.0.0 (2023-04-06)","type":"extras"},{"doc":"","ref":"changelog.html#changelog","title":"Changelog - Changelog","type":"extras"}]